@@ -1264,16 +1264,17 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
   unsigned char      *data, *ptr, *end;
   int                 pc, c, i, j, k, ncolors, cpp, comment, transp, quote,
                       context, len, done;
-  char                line[65536], s[65536], tok[65536], col[65536];
+  char                *line, s[256], tok[128], col[256];
   XColor              xcol;
+  int                 lsz = 256;
   struct _cmap
     {
-      unsigned char       str[8];
+      unsigned char       str[6];
       unsigned char       transp;
-      int                 r, g, b;
+      short                 r, g, b;
     }
                      *cmap;
-  int                 lookup[256][256];
+  short                 lookup[128 - 32][128 - 32];
 
   transp = 0;
   done = 0;
@@ -1291,6 +1292,7 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
   comment = 0;
   quote = 0;
   context = 0;
+  line = malloc(lsz);
   while (!done)
     {
       pc = c;
@@ -1319,29 +1321,42 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 		{
 		  /* Header */
 		  sscanf(line, "%i %i %i %i", w, h, &ncolors, &cpp);
-		  if (cpp > 7)
+                  if (ncolors > 32766)
 		    {
-		      fprintf(stderr, "IMLIB ERROR: XPM files with characters per pixel > 7 not supported\n");
+		      fprintf(stderr, "IMLIB ERROR: XPM files wth colors > 32766 not supported\n");
+		      free(line);
+		      return NULL;
+		    }
+		  if (cpp > 5)
+		    {
+		      fprintf(stderr, "IMLIB ERROR: XPM files with characters per pixel > 5 not supported\n");
+		      free(line);
 		      return NULL;
 		    }
 		  if (*w > 32767)
 		    {
 		      fprintf(stderr, "IMLIB ERROR: Image width > 32767 pixels for file\n");
+		      free(line);
 		      return NULL;
 		    }
 		  if (*h > 32767)
 		    {
 		      fprintf(stderr, "IMLIB ERROR: Image height > 32767 pixels for file\n");
+		      free(line);
 		      return NULL;
 		    }
 		  cmap = malloc(sizeof(struct _cmap) * ncolors);
 
 		  if (!cmap)
-		    return NULL;
+		    {
+		      free(line);
+		      return NULL;
+		    }
 		  data = malloc(*w ** h * 3);
 		  if (!data)
 		    {
 		      free(cmap);
+		      free(line);
 		      return NULL;
 		    }
 		  ptr = data;
@@ -1385,7 +1400,8 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 				    {
 				      if (col[0])
 					strcat(col, " ");
-				      strcat(col, s);
+                                      if (strlen(col) + strlen(s) < sizeof(col))
+					strcat(col, s);
 				    }
 				  if (col[0])
 				    {
@@ -1432,10 +1448,10 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 		    {
 		      if (cpp == 1)
 			for (i = 0; i < ncolors; i++)
-			  lookup[(int)cmap[i].str[0]][(int)cmap[i].str[1]] = i;
+			  lookup[(int)cmap[i].str[0] - 32][0] = i;
 		      if (cpp == 2)
 			for (i = 0; i < ncolors; i++)
-			  lookup[(int)cmap[i].str[0]][(int)cmap[i].str[1]] = i;
+			  lookup[(int)cmap[i].str[0] - 32][(int)cmap[i].str[1] - 32] = i;
 		      context++;
 		    }
 		}
@@ -1454,7 +1470,7 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 			  for (i = 0; ((i < 65536) && (ptr < end) && (line[i])); i++)
 			    {
 			      col[0] = line[i];
-			      if (cmap[lookup[(int)col[0]][0]].transp)
+			      if (cmap[lookup[(int)col[0] - 32][0]].transp)
 				{
 				  *ptr++ = 255;
 				  *ptr++ = 0;
@@ -1462,9 +1478,9 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 				}
 			      else
 				{
-				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].r;
-				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].g;
-				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].b;
+				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].r;
+				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].g;
+				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].b;
 				}
 			    }
 			}
@@ -1473,9 +1489,9 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 			  for (i = 0; ((i < 65536) && (ptr < end) && (line[i])); i++)
 			    {
 			      col[0] = line[i];
-			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].r;
-			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].g;
-			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].b;
+			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].r;
+			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].g;
+			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].b;
 			    }
 			}
 		    }
@@ -1487,7 +1503,7 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 			    {
 			      col[0] = line[i++];
 			      col[1] = line[i];
-			      if (cmap[lookup[(int)col[0]][(int)col[1]]].transp)
+			      if (cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].transp)
 				{
 				  *ptr++ = 255;
 				  *ptr++ = 0;
@@ -1495,9 +1511,9 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 				}
 			      else
 				{
-				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].r;
-				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].g;
-				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].b;
+				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].r;
+				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].g;
+				  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].b;
 				}
 			    }
 			}
@@ -1507,9 +1523,9 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 			    {
 			      col[0] = line[i++];
 			      col[1] = line[i];
-			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].r;
-			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].g;
-			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].b;
+			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].r;
+			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].g;
+			      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].b;
 			    }
 			}
 		    }
@@ -1572,13 +1588,19 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
 		}
 	    }
 	}
-      /* Scan in line from XPM file (limit line length 65k) */
-      if (i < 65536)
+      /* Scan in line from XPM file */
+      if ((!comment) && (quote) && (c != '"'))
 	{
-	  if ((!comment) && (quote) && (c != '"'))
-	    {
-	      line[i++] = c;
-	    }
+	  if (c < 32)
+	    c = 32;
+	  else if (c > 127)
+	    c = 127;
+	  line[i++] = c;
+	}
+      if (i >= lsz)
+	{
+	  lsz += 256;
+	  line = realloc(line, lsz);
 	}
       if ((ptr) && ((ptr - data) >= *w ** h * 3))
 	done = 1;
@@ -1588,6 +1610,7 @@ _LoadXPM(ImlibData * id, FILE *file, int *w, int *h, int *t)
   else
     *t = 0;
   free(cmap);
+  free(line);
   return data;
 }
 
