@@ -84,6 +84,7 @@ Imlib_init(Display * disp)
   int                 loadpal;
   int                 vis;
   int                 newcm;
+  char               *old_locale;
 
   fprintf(stderr, "Imlib Init\n");
   fflush(stderr);
@@ -180,7 +181,8 @@ Imlib_init(Display * disp)
   homedir = getenv("HOME");
   snprintf(s, sizeof(s), "%s/.imrc", homedir);
 
-  setlocale(LC_ALL, "C");
+  old_locale = strdup(setlocale(LC_NUMERIC, NULL));
+  setlocale(LC_NUMERIC, "C");
   f = fopen(s, "r");
   if (!f)
     f = fopen(SYSTEM_IMRC, "r");
@@ -349,7 +351,10 @@ Imlib_init(Display * disp)
 	}
       fclose(f);
     }
-  setlocale(LC_ALL, "");
+  setlocale(LC_NUMERIC, old_locale);
+  if (old_locale)
+    free(old_locale);
+  
   /* list all visuals for the default screen */
   xvi.screen = id->x.screen;
   xvir = XGetVisualInfo(disp, VisualScreenMask, &xvi, &num);
@@ -625,6 +630,7 @@ Imlib_init_with_params(Display * disp, ImlibInitParams * p)
   int                 loadpal;
   int                 vis;
   int                 newcm;
+  char               *old_locale;
 
   palfile = NULL;
   if (!disp)
@@ -717,7 +723,8 @@ Imlib_init_with_params(Display * disp, ImlibInitParams * p)
   id->num_colors = 0;
   homedir = getenv("HOME");
   snprintf(s, sizeof(s), "%s/.imrc", homedir);
-  setlocale(LC_ALL, "C");
+  old_locale = strdup(setlocale(LC_NUMERIC, NULL));
+  setlocale(LC_NUMERIC, "C");
   f = fopen(s, "r");
   if (!f)
     f = fopen(SYSTEM_IMRC, "r");
@@ -886,7 +893,10 @@ Imlib_init_with_params(Display * disp, ImlibInitParams * p)
 	}
       fclose(f);
     }
-  setlocale(LC_ALL, "");
+  setlocale(LC_NUMERIC, old_locale);
+  if (old_locale)
+    free(old_locale);
+  
   if (p)
     {
       if (p->flags & PARAMS_VISUALID)
