@@ -1102,16 +1102,16 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
   int                 c, i, j, k, ncolors, cpp, comment, transp, quote, context,
                       len, count, done;
   int                 w, h;
-  char               *line, s[65536], tok[65536], col[65536];
+  char               *line, s[256], tok[128], col[256];
   XColor              xcol;
   struct _cmap
     {
-      char                str[8];
+      char                str[6];
       char                transp;
-      int                 r, g, b;
+      short                 r, g, b;
     }
                      *cmap;
-  int                 lookup[128][128];
+  short                 lookup[128 - 32][128 - 32];
 
   if (!data)
     return NULL;
@@ -1138,21 +1138,27 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 	{
 	  /* Header */
 	  sscanf(line, "%i %i %i %i", &w, &h, &ncolors, &cpp);
-	  if (cpp > 7)
+	  if (ncolors > 32766)
 	    {
-	      fprintf(stderr, "gdk_imlib ERROR: XPM data with characters per pixel > 7 not supported\n");
+	      fprintf(stderr, "IMLIB ERROR: XPM data wth colors > 32766 not supported\n");
+	      free(im);
+	      return NULL;
+	    }
+	  if (cpp > 5)
+	    {
+	      fprintf(stderr, "IMLIB ERROR: XPM data with characters per pixel > 5 not supported\n");
 	      free(im);
 	      return NULL;
 	    }
 	  if (w > 32767)
 	    {
-	      fprintf(stderr, "gdk_imlib ERROR: Image width > 32767 pixels for data\n");
+	      fprintf(stderr, "IMLIB ERROR: Image width > 32767 pixels for data\n");
 	      free(im);
 	      return NULL;
 	    }
 	  if (h > 32767)
 	    {
-	      fprintf(stderr, "gdk_imlib ERROR: Image height > 32767 pixels for data\n");
+	      fprintf(stderr, "IMLIB ERROR: Image height > 32767 pixels for data\n");
 	      free(im);
 	      return NULL;
 	    }
@@ -1173,10 +1179,8 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 	      return NULL;
 	    }
 	  im->alpha_data = NULL;
-	  snprintf(s, sizeof(s), "creat_%x_%x", (int)time(NULL), (int)rand());
-	  im->filename = malloc(strlen(s) + 1);
-	  if (im->filename)
-	    strcpy(im->filename, s);
+          snprintf (s, sizeof (s), "creat_%lx_%x", time(NULL), rand());
+	  im->filename = strdup(s);
 	  im->width = 0;
 	  im->height = 0;
 	  im->border.left = 0;
@@ -1205,9 +1209,9 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 	}
       else if (context == 1)
 	{
-	  int                 colptr;
+          int                 colptr;
 	  int                 hascolor, iscolor;
-
+	  
 	  /* Color Table */
 	  if (j < ncolors)
 	    {
@@ -1237,9 +1241,9 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 			  if (k >= len)
 			    {
 			      int                 ls;
-
+			      
 			      ls = strlen(s);
-
+			      
 			      if (col[0] && colptr < sizeof(col))
 				{
 				  strcpy(col + colptr, " ");
@@ -1250,7 +1254,7 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 				  strcpy(col + colptr, s);
 				  colptr += ls;
 				}
-
+			      
 			    }
 			  if (col[0])
 			    {
@@ -1287,9 +1291,8 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 		      else
 			{
 			  int                 ls;
-
+			  
 			  ls = strlen(s);
-
 			  if (col[0] && colptr < sizeof(col))
 			    {
 			      strcpy(col + colptr, " ");
@@ -1309,10 +1312,10 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 	    {
 	      if (cpp == 1)
 		for (i = 0; i < ncolors; i++)
-		  lookup[(int)cmap[i].str[0]][(int)cmap[i].str[1]] = i;
-	      if (cpp == 2)
+		  lookup[(int)cmap[i].str[0] - 32][0] = i;
+	      else if (cpp == 2)
 		for (i = 0; i < ncolors; i++)
-		  lookup[(int)cmap[i].str[0]][(int)cmap[i].str[1]] = i;
+		  lookup[(int)cmap[i].str[0] - 32][(int)cmap[i].str[1] - 32] = i;
 	      context++;
 	    }
 	}
@@ -1330,7 +1333,7 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 		  for (i = 0; ((i < 65536) && (ptr < end) && (line[i])); i++)
 		    {
 		      col[0] = line[i];
-		      if (cmap[lookup[(int)col[0]][0]].transp)
+		      if (cmap[lookup[(int)col[0] - 32][0]].transp)
 			{
 			  *ptr++ = 255;
 			  *ptr++ = 0;
@@ -1338,9 +1341,9 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 			}
 		      else
 			{
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].r;
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].g;
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].b;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].r;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].g;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].b;
 			}
 		    }
 		}
@@ -1349,9 +1352,9 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 		  for (i = 0; ((i < 65536) && (ptr < end) && (line[i])); i++)
 		    {
 		      col[0] = line[i];
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].r;
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].g;
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].b;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].r;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].g;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].b;
 		    }
 		}
 	    }
@@ -1363,7 +1366,7 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 		    {
 		      col[0] = line[i++];
 		      col[1] = line[i];
-		      if (cmap[lookup[(int)col[0]][(int)col[1]]].transp)
+		      if (cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].transp)
 			{
 			  *ptr++ = 255;
 			  *ptr++ = 0;
@@ -1371,9 +1374,9 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 			}
 		      else
 			{
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].r;
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].g;
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].b;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].r;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].g;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].b;
 			}
 		    }
 		}
@@ -1383,9 +1386,9 @@ Imlib_create_image_from_xpm_data(ImlibData * id, char **data)
 		    {
 		      col[0] = line[i++];
 		      col[1] = line[i];
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].r;
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].g;
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].b;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].r;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].g;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].b;
 		    }
 		}
 	    }
@@ -1472,7 +1475,7 @@ int
 Imlib_data_to_pixmap(ImlibData * id, char **data, Pixmap * pmap, Pixmap * mask)
 {
   ImlibImage         *im;
-
+  
   im = Imlib_create_image_from_xpm_data(id, data);
   if (!im)
     {
