@@ -11,4 +11,7 @@
 #undef HAVE_LIBGIF
 #undef HAVE_LIBPNG
 
+#undef WITH_SYMBOL_UNDERSCORE
+
 #undef HAVE_STDARGS
+#undef USE_GMODULE
