@@ -116,6 +116,7 @@ _gdk_imlib_calc_map_tables(GdkImlibImage * im)
       map->bmap[i] = (unsigned char)v;
     }
   _gdk_imlib_dirty_pixmaps(im);
+  _gdk_imlib_clean_caches();
 }
 
 gint
@@ -275,6 +276,7 @@ gdk_imlib_set_image_red_curve(GdkImlibImage * im, unsigned char *mod)
   for (i = 0; i < 256; i++)
     map->rmap[i] = mod[i];
   _gdk_imlib_dirty_pixmaps(im);
+  _gdk_imlib_clean_caches();
   im->mod.contrast = 257;
 }
 
@@ -309,6 +311,7 @@ gdk_imlib_set_image_green_curve(GdkImlibImage * im, unsigned char *mod)
   for (i = 0; i < 256; i++)
     map->gmap[i] = mod[i];
   _gdk_imlib_dirty_pixmaps(im);
+  _gdk_imlib_clean_caches();
   im->mod.contrast = 257;
 }
 
@@ -343,6 +346,7 @@ gdk_imlib_set_image_blue_curve(GdkImlibImage * im, unsigned char *mod)
   for (i = 0; i < 256; i++)
     map->bmap[i] = mod[i];
   _gdk_imlib_dirty_pixmaps(im);
+  _gdk_imlib_clean_caches();
   im->mod.contrast = 257;
 }
 
@@ -441,6 +445,7 @@ gdk_imlib_apply_modifiers_to_rgb(GdkImlibImage * im)
   _gdk_imlib_calc_map_tables(im);
   _gdk_imlib_dirty_images(im);
   _gdk_imlib_dirty_pixmaps(im);
+  _gdk_imlib_clean_caches();
 }
 
 void
@@ -518,6 +523,7 @@ gdk_imlib_crop_image(GdkImlibImage * im, gint x, gint y, gint w, gint h)
   im->rgb_height = h;
   _gdk_imlib_dirty_images(im);
   _gdk_imlib_dirty_pixmaps(im);
+  _gdk_imlib_clean_caches();
 }
 
 GdkImlibImage      *
@@ -603,10 +609,8 @@ gdk_imlib_crop_and_clone_image(GdkImlibImage * im, int x, int y, int w, int h)
   s = malloc(strlen(im->filename) + 320);
   if (s)
     {
-      g_snprintf(s, sizeof(s), "%s_%x_%x", im->filename, (int)time(NULL), (int)rand());
-      im2->filename = malloc(strlen(s) + 1);
-      if (im2->filename)
-	strcpy(im2->filename, s);
+      g_snprintf(s, strlen(im->filename) + 320, "%s_%lx_%x", im->filename, time(NULL), rand());
+      im2->filename = strdup(s);
       free(s);
     }
   else
@@ -643,6 +647,7 @@ gdk_imlib_changed_image(GdkImlibImage * im)
     return;
   _gdk_imlib_dirty_images(im);
   _gdk_imlib_dirty_pixmaps(im);
+  _gdk_imlib_clean_caches();
 }
 
 void
@@ -775,6 +780,7 @@ gdk_imlib_flip_image_horizontal(GdkImlibImage * im)
   im->border.right = w3;
   _gdk_imlib_dirty_images(im);
   _gdk_imlib_dirty_pixmaps(im);
+  _gdk_imlib_clean_caches();
 }
 
 void
@@ -813,6 +819,7 @@ gdk_imlib_flip_image_vertical(GdkImlibImage * im)
   im->border.bottom = w3;
   _gdk_imlib_dirty_images(im);
   _gdk_imlib_dirty_pixmaps(im);
+  _gdk_imlib_clean_caches();
 }
 
 void
@@ -856,6 +863,7 @@ gdk_imlib_rotate_image(GdkImlibImage * im, gint d)
   im->border.right = w3;
   _gdk_imlib_dirty_images(im);
   _gdk_imlib_dirty_pixmaps(im);
+  _gdk_imlib_clean_caches();
 }
 
 GdkImlibImage      *
@@ -885,10 +893,8 @@ gdk_imlib_create_image_from_data(unsigned char *data, unsigned char *alpha, gint
   memcpy(im->rgb_data, data, im->rgb_width * im->rgb_height * 3);
 /*      im->alpha_data=alpha; */
   im->alpha_data = NULL;
-  g_snprintf(s, sizeof(s), "creat_%x_%x", (int)time(NULL), (int)rand());
-  im->filename = malloc(strlen(s) + 1);
-  if (im->filename)
-    strcpy(im->filename, s);
+  g_snprintf(s, sizeof(s), "creat_%lx_%x", time(NULL), rand());
+  im->filename = strdup(s);
   im->width = 0;
   im->height = 0;
   im->shape_color.r = -1;
@@ -958,10 +964,8 @@ gdk_imlib_clone_image(GdkImlibImage * im)
   s = malloc(strlen(im->filename) + 320);
   if (s)
     {
-      g_snprintf(s, sizeof(s), "%s_%x_%x", im->filename, (int)time(NULL), (int)rand());
-      im2->filename = malloc(strlen(s) + 1);
-      if (im2->filename)
-	strcpy(im2->filename, s);
+      g_snprintf(s, strlen(im->filename) + 320, "%s_%lx_%x", im->filename, time(NULL), rand());
+      im2->filename = strdup(s);
       free(s);
     }
   else
@@ -1141,10 +1145,8 @@ gdk_imlib_clone_scaled_image(GdkImlibImage * im, int w, int h)
   s = malloc(strlen(im->filename) + 320);
   if (s)
     {
-      g_snprintf(s, sizeof(s), "%s_%x_%x_%x_%x", im->filename, (int)time(NULL), w, h, (int)rand());
-      im2->filename = malloc(strlen(s) + 1);
-      if (im2->filename)
-	strcpy(im2->filename, s);
+      g_snprintf(s, strlen(im->filename) + 320, "%s_%lx_%x_%x_%x", im->filename, time(NULL), w, h, rand());
+      im2->filename = strdup(s);
       free(s);
     }
   else
@@ -1187,16 +1189,16 @@ gdk_imlib_create_image_from_xpm_data(char **data)
   int                 c, i, j, k, ncolors, cpp, comment, transp, quote, context,
                       len, count, done;
   int                 w, h;
-  char               *line, s[65536], tok[65536], col[65536];
+  char               *line, s[256], tok[128], col[256];
   XColor              xcol;
   struct _cmap
     {
-      char                str[8];
+      char                str[6];
       char                transp;
-      int                 r, g, b;
+      short               r, g, b;
     }
-                     *cmap;
-  int                 lookup[128][128];
+  *cmap;
+  short                 lookup[128 - 32][128 - 32];
 
   g_return_val_if_fail (data != NULL, NULL);
   
@@ -1225,9 +1227,15 @@ gdk_imlib_create_image_from_xpm_data(char **data)
 	{
 	  /* Header */
 	  sscanf(line, "%i %i %i %i", &w, &h, &ncolors, &cpp);
-	  if (cpp > 7)
+	  if (ncolors > 32766)
 	    {
-	      fprintf(stderr, "gdk_imlib ERROR: XPM data with characters per pixel > 7 not supported\n");
+	      fprintf(stderr, "gdk_imlib ERROR: XPM data wth colors > 32766 not supported\n");
+	      free(im);
+	      return NULL;
+	    }
+	  if (cpp > 5)
+	    {
+	      fprintf(stderr, "gdk_imlib ERROR: XPM data with characters per pixel > 5 not supported\n");
 	      free(im);
 	      return NULL;
 	    }
@@ -1260,7 +1268,7 @@ gdk_imlib_create_image_from_xpm_data(char **data)
 	      return NULL;
 	    }
 	  im->alpha_data = NULL;
-	  g_snprintf (s, sizeof (s), "creat_%x_%x", (int)time(NULL), (int)rand());
+	  g_snprintf (s, sizeof (s), "creat_%lx_%x", time(NULL), rand());
 	  im->filename = strdup (s);
 	  im->width = 0;
 	  im->height = 0;
@@ -1394,10 +1402,10 @@ gdk_imlib_create_image_from_xpm_data(char **data)
 	    {
 	      if (cpp == 1)
 		for (i = 0; i < ncolors; i++)
-		  lookup[(int)cmap[i].str[0]][(int)cmap[i].str[1]] = i;
-	      if (cpp == 2)
+		  lookup[(int)cmap[i].str[0] - 32][0] = i;
+	      else if (cpp == 2)
 		for (i = 0; i < ncolors; i++)
-		  lookup[(int)cmap[i].str[0]][(int)cmap[i].str[1]] = i;
+		  lookup[(int)cmap[i].str[0] - 32][(int)cmap[i].str[1] - 32] = i;
 	      context++;
 	    }
 	}
@@ -1415,7 +1423,7 @@ gdk_imlib_create_image_from_xpm_data(char **data)
 		  for (i = 0; ((i < 65536) && (ptr < end) && (line[i])); i++)
 		    {
 		      col[0] = line[i];
-		      if (cmap[lookup[(int)col[0]][0]].transp)
+		      if (cmap[lookup[(int)col[0] - 32][0]].transp)
 			{
 			  *ptr++ = 255;
 			  *ptr++ = 0;
@@ -1423,9 +1431,9 @@ gdk_imlib_create_image_from_xpm_data(char **data)
 			}
 		      else
 			{
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].r;
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].g;
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].b;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].r;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].g;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].b;
 			}
 		    }
 		}
@@ -1434,9 +1442,9 @@ gdk_imlib_create_image_from_xpm_data(char **data)
 		  for (i = 0; ((i < 65536) && (ptr < end) && (line[i])); i++)
 		    {
 		      col[0] = line[i];
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].r;
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].g;
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][0]].b;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].r;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].g;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][0]].b;
 		    }
 		}
 	    }
@@ -1448,7 +1456,7 @@ gdk_imlib_create_image_from_xpm_data(char **data)
 		    {
 		      col[0] = line[i++];
 		      col[1] = line[i];
-		      if (cmap[lookup[(int)col[0]][(int)col[1]]].transp)
+		      if (cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].transp)
 			{
 			  *ptr++ = 255;
 			  *ptr++ = 0;
@@ -1456,9 +1464,9 @@ gdk_imlib_create_image_from_xpm_data(char **data)
 			}
 		      else
 			{
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].r;
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].g;
-			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].b;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].r;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].g;
+			  *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].b;
 			}
 		    }
 		}
@@ -1468,9 +1476,9 @@ gdk_imlib_create_image_from_xpm_data(char **data)
 		    {
 		      col[0] = line[i++];
 		      col[1] = line[i];
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].r;
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].g;
-		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0]][(int)col[1]]].b;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].r;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].g;
+		      *ptr++ = (unsigned char)cmap[lookup[(int)col[0] - 32][(int)col[1] - 32]].b;
 		    }
 		}
 	    }
@@ -1546,8 +1554,13 @@ gdk_imlib_create_image_from_xpm_data(char **data)
       im->shape_color.g = 0;
       im->shape_color.b = 255;
     }
-  g_snprintf(s, sizeof(s),"%i.%i.%i.%s.%s.%s", data, im->rgb_width, im->rgb_height, data[0], data[1], data [3]);
-  im->filename = g_strdup(s);
+
+  i = 100 + strlen (data[0]) + strlen (data[1]) + strlen (data[3]);
+  im->filename = (char *) malloc (i);
+  if (im->filename)
+    g_snprintf (im->filename, i, "%p.%i.%i.%s.%s.%s", data, im->rgb_width,
+		im->rgb_height, data[0], data[1], data [3]);
+
   if (id->cache.on_image)
     _gdk_imlib_add_image(im, im->filename);
   _gdk_imlib_calc_map_tables(im);
