@@ -1,3 +1,25 @@
+#include <config.h>
+#ifndef GNOMELOCALEDIR
+#define GNOMELOCALEDIR "/usr/share/locale"
+#endif
+#ifdef ENABLE_NLS
+#    include <libintl.h>
+#    define _(String) gettext (String)
+#    ifdef gettext_noop
+#        define N_(String) gettext_noop (String)
+#    else
+#        define N_(String) (String)
+#    endif
+#else
+/* Stubs that do something close enough.  */
+#    define textdomain(String) (String)
+#    define gettext(String) (String)
+#    define dgettext(Domain,Message) (Message)
+#    define dcgettext(Domain,Message,Type) (Message)
+#    define bindtextdomain(Domain,Directory) (Domain)
+#    define _(String) (String)
+#    define N_(String) (String)
+#endif
 
 #include <gdk/gdk.h>
 #include <gdk_imlib.h>
@@ -10,6 +32,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <locale.h>
 
 GtkWidget          *note = NULL;
 GtkWidget          *pg1 = NULL;
@@ -159,7 +182,7 @@ make_win()
 		      GTK_SIGNAL_FUNC(kill_win), NULL);
    gtk_container_set_border_width(GTK_CONTAINER(win), 4);
    gtk_window_set_policy(GTK_WINDOW(win), 0, 0, 1);
-   gtk_window_set_title(GTK_WINDOW(win), "Imlib Config Editor");
+   gtk_window_set_title(GTK_WINDOW(win), _("Imlib Config Editor"));
    gtk_window_set_wmclass(GTK_WINDOW(win), "Imlib Config Editor", "Config");
 
    box1 = gtk_vbox_new(FALSE, 0);
@@ -395,7 +418,8 @@ load_pal(char *f)
    int                 r, g, b;
    char                s[1024];
 
-   ff = fopen(f, "r");
+  setlocale(LC_ALL, "C");
+  ff = fopen(f, "r");
    if (ff)
      {
 	color.number = 0;
@@ -423,6 +447,7 @@ load_pal(char *f)
 	  }
 	fclose(ff);
      }
+  setlocale(LC_ALL, "");
 }
 
 void
@@ -431,13 +456,15 @@ save_pal(char *f)
    FILE               *ff;
    int                 i;
 
-   ff = fopen(f, "w");
+  setlocale(LC_ALL, "C");
+  ff = fopen(f, "w");
    if (ff)
      {
 	for (i = 0; i < color.number; i++)
 	   fprintf(ff, "0x%x 0x%x 0x%x\n", color.color[i][0], color.color[i][1], color.color[i][2]);
 	fclose(ff);
      }
+  setlocale(LC_ALL, "");
 }
 
 void
@@ -716,7 +743,8 @@ load_imrc()
 
    h = getenv("HOME");
    g_snprintf(s, 2048, "%s/.imrc", h);
-   f = fopen(s, "r");
+  setlocale(LC_ALL, "C");
+  f = fopen(s, "r");
    if (!f)
      {
 	h = gdk_imlib_get_sysconfig();
@@ -724,6 +752,7 @@ load_imrc()
 	free(h);
      }
    read_imrc(f);
+  setlocale(LC_ALL, "");
 }
 
 void
@@ -735,7 +764,9 @@ save_user(GtkWidget * widget, gpointer * data)
    h = getenv("HOME");
    g_snprintf(s, 2048, "%s/.imrc", h);
    f = fopen(s, "w");
-   write_imrc(f);
+  setlocale(LC_ALL, "C");
+  write_imrc(f);
+  setlocale(LC_ALL, "");
 }
 
 void
@@ -747,7 +778,10 @@ save_sys(GtkWidget * widget, gpointer * data)
    h = gdk_imlib_get_sysconfig();
    f = fopen(h, "w");
    free(h);
-   write_imrc(f);
+  setlocale(LC_ALL, "C");
+  write_imrc(f);
+  setlocale(LC_ALL, "");
+  
 }
 
 void
@@ -897,7 +931,7 @@ add_sliders(GtkWidget * w)
    gtk_widget_show(i2);
    i3 = gtk_pixmap_new(contrast_pmap, contrast_mask);
    gtk_widget_show(i3);
-   frame = gtk_aspect_frame_new("Base Levels", 0.5, 0.5, 0.0, TRUE);
+   frame = gtk_aspect_frame_new(_("Base Levels"), 0.5, 0.5, 0.0, TRUE);
    gtk_box_pack_start(GTK_BOX(box1), frame, FALSE, FALSE, 4);
    gtk_widget_show(frame);
    box = gtk_vbox_new(TRUE, 0);
@@ -913,7 +947,7 @@ add_sliders(GtkWidget * w)
    gtk_widget_show(i2);
    i3 = gtk_pixmap_new(contrast_pmap, contrast_mask);
    gtk_widget_show(i3);
-   frame = gtk_aspect_frame_new("Red Levels", 0.5, 0.5, 0.0, TRUE);
+   frame = gtk_aspect_frame_new(_("Red Levels"), 0.5, 0.5, 0.0, TRUE);
    gtk_box_pack_start(GTK_BOX(box1), frame, FALSE, FALSE, 4);
    gtk_widget_show(frame);
    box = gtk_vbox_new(TRUE, 0);
@@ -929,7 +963,7 @@ add_sliders(GtkWidget * w)
    gtk_widget_show(i2);
    i3 = gtk_pixmap_new(contrast_pmap, contrast_mask);
    gtk_widget_show(i3);
-   frame = gtk_aspect_frame_new("Green Levels", 0.5, 0.5, 0.0, TRUE);
+   frame = gtk_aspect_frame_new(_("Green Levels"), 0.5, 0.5, 0.0, TRUE);
    gtk_box_pack_start(GTK_BOX(box1), frame, FALSE, FALSE, 4);
    gtk_widget_show(frame);
    box = gtk_vbox_new(TRUE, 0);
@@ -945,7 +979,7 @@ add_sliders(GtkWidget * w)
    gtk_widget_show(i2);
    i3 = gtk_pixmap_new(contrast_pmap, contrast_mask);
    gtk_widget_show(i3);
-   frame = gtk_aspect_frame_new("Blue Levels", 0.5, 0.5, 0.0, TRUE);
+   frame = gtk_aspect_frame_new(_("Blue Levels"), 0.5, 0.5, 0.0, TRUE);
    gtk_box_pack_start(GTK_BOX(box1), frame, FALSE, FALSE, 4);
    gtk_widget_show(frame);
    box = gtk_vbox_new(TRUE, 0);
@@ -972,7 +1006,7 @@ add_palsel(GtkWidget * w, char *title, char **value)
    gtk_box_pack_start(GTK_BOX(box), box2, TRUE, TRUE, 0);
    gtk_widget_show(box2);
 
-   filesel = gtk_file_selection_new("Select Palette File");
+   filesel = gtk_file_selection_new(_("Select Palette File"));
    gtk_signal_connect(GTK_OBJECT(filesel), "destroy",
 		      GTK_SIGNAL_FUNC(showhide), filesel);
    gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->cancel_button),
@@ -981,7 +1015,7 @@ add_palsel(GtkWidget * w, char *title, char **value)
    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button),
 		      "clicked", GTK_SIGNAL_FUNC(read_file), filesel);
 
-   button = gtk_button_new_with_label("Select");
+   button = gtk_button_new_with_label(_("Select"));
    gtk_box_pack_start(GTK_BOX(box2), button, FALSE, FALSE, 0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(pal_load), filesel);
@@ -996,7 +1030,7 @@ add_palsel(GtkWidget * w, char *title, char **value)
    gtk_signal_connect(GTK_OBJECT(text), "changed", GTK_SIGNAL_FUNC(change_str), text);
    pal_text = text;
 
-   button = gtk_button_new_with_label("Save");
+   button = gtk_button_new_with_label(_("Save"));
    gtk_box_pack_start(GTK_BOX(box2), button, FALSE, FALSE, 0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(pal_save), filesel);
@@ -1028,12 +1062,12 @@ add_palsel(GtkWidget * w, char *title, char **value)
    gtk_box_pack_start(GTK_BOX(box), box3, TRUE, TRUE, 0);
    gtk_widget_show(box3);
 
-   button = gtk_button_new_with_label("Copy To");
+   button = gtk_button_new_with_label(_("Copy To"));
    gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(col_copy), NULL);
 
-   button = gtk_button_new_with_label("Spread To");
+   button = gtk_button_new_with_label(_("Spread To"));
    gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(col_spread), NULL);
@@ -1042,12 +1076,12 @@ add_palsel(GtkWidget * w, char *title, char **value)
    gtk_box_pack_start(GTK_BOX(box), box3, TRUE, TRUE, 0);
    gtk_widget_show(box3);
 
-   button = gtk_button_new_with_label("Set Last");
+   button = gtk_button_new_with_label(_("Set Last"));
    gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(col_setlast), NULL);
 
-   button = gtk_button_new_with_label("Delete");
+   button = gtk_button_new_with_label(_("Delete"));
    gtk_box_pack_start(GTK_BOX(box3), button, TRUE, TRUE, 0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(col_del), NULL);
@@ -1287,12 +1321,12 @@ add_loadsave()
    gtk_box_pack_start(GTK_BOX(list), box2, FALSE, FALSE, 8);
    gtk_widget_show(box2);
 
-   button = gtk_button_new_with_label("Save User Config");
+   button = gtk_button_new_with_label(_("Save User Config"));
    gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(save_user), NULL);
 
-   button = gtk_button_new_with_label("Save System Config");
+   button = gtk_button_new_with_label(_("Save System Config"));
    gtk_box_pack_start(GTK_BOX(box2), button, TRUE, TRUE, 0);
    gtk_widget_show(button);
    gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(save_sys), NULL);
@@ -1416,6 +1450,9 @@ main(int argc, char **argv)
    gdk_imlib_init();
    list_vis();
 
+   bindtextdomain(PACKAGE, GNOMELOCALEDIR);
+   textdomain(PACKAGE);
+
    gtk_widget_push_visual(gdk_imlib_get_visual());
    gtk_widget_push_colormap(gdk_imlib_get_colormap());
    color.pmap = NULL;
@@ -1429,42 +1466,42 @@ main(int argc, char **argv)
    init_conf();
    load_imrc();
    make_win();
-   add_note("Colors", "Rendering", "Display", "Color Correction");
-   add_palsel(pg1, "Select Palette", &conf.palettefile);
-   add_onoff(pg1, "Force Palette", &conf.paletteoverride);
-   add_onoff(pg1, "Dither in Palette Mode", &conf.dither);
-   add_onoff(pg1, "Fast Remapping in Palette Mode", &conf.remap);
-   add_onoff(pg2, "High Quality in 15/16bpp", &conf.highquality);
-   add_onoff_value(pg2, "Limit Maximum Shared memory Size (bytes)",
+   add_note(_("Colors"), _("Rendering"), _("Display"), _("Color Correction"));
+   add_palsel(pg1, _("Select Palette"), &conf.palettefile);
+   add_onoff(pg1, _("Force Palette"), &conf.paletteoverride);
+   add_onoff(pg1, _("Dither in Palette Mode"), &conf.dither);
+   add_onoff(pg1, _("Fast Remapping in Palette Mode"), &conf.remap);
+   add_onoff(pg2, _("High Quality in 15/16bpp"), &conf.highquality);
+   add_onoff_value(pg2, _("Limit Maximum Shared memory Size (bytes)"),
 		   &conf.shmmax, &conf.shmmaxsize,
-		   "256 Kb", 256 * 1024,
-		   "512 Kb", 512 * 1024,
-		   "1   Mb", 1 * 1024 * 1024,
-		   "2   Mb", 2 * 1024 * 1024,
-		   "3   Mb", 3 * 1024 * 1024,
-		   "4   Mb", 4 * 1024 * 1024);
-   add_onoff(pg2, "Use Ordered Dithering (Faster)", &conf.ordered);
-   add_onoff(pg2, "MIT-SHM Shared Memory Enabled", &conf.mitshm);
-   add_onoff(pg2, "Shared Pixmaps", &conf.sharedpixmap);
-   add_onoff(pg2, "Fast Render", &conf.fastrender);
-   add_onoff(pg2, "Imagemagick & NETPBM Fallback", &conf.fallback);
-   add_onoff_value(pg2, "Image Cache (bytes)",
+		   _("256 Kb"), 256 * 1024,
+		   _("512 Kb"), 512 * 1024,
+		   _("1   Mb"), 1 * 1024 * 1024,
+		   _("2   Mb"), 2 * 1024 * 1024,
+		   _("3   Mb"), 3 * 1024 * 1024,
+		   _("4   Mb"), 4 * 1024 * 1024);
+   add_onoff(pg2, _("Use Ordered Dithering (Faster)"), &conf.ordered);
+   add_onoff(pg2, _("MIT-SHM Shared Memory Enabled"), &conf.mitshm);
+   add_onoff(pg2, _("Shared Pixmaps"), &conf.sharedpixmap);
+   add_onoff(pg2, _("Fast Render"), &conf.fastrender);
+   add_onoff(pg2, _("Imagemagick & NETPBM Fallback"), &conf.fallback);
+   add_onoff_value(pg2, _("Image Cache (bytes)"),
 		   &conf.imagecache, &conf.imagecachesize,
-		   "512 Kb", 512 * 1024,
-		   "1 Mb", 1024 * 1024,
-		   "2 Mb", 2 * 1024 * 1024,
-		   "4 Mb", 4 * 1024 * 1024,
-		   "6 Mb", 6 * 1024 * 1024,
-		   "10 Mb", 10 * 1024 * 1024);
-   add_onoff_value(pg2, "Pixmap Cache (bits)",
+		   _("512 Kb"), 512 * 1024,
+		   _("1 Mb"), 1024 * 1024,
+		   _("2 Mb"), 2 * 1024 * 1024,
+		   _("4 Mb"), 4 * 1024 * 1024,
+		   _("6 Mb"), 6 * 1024 * 1024,
+		   _("10 Mb"), 10 * 1024 * 1024);
+   add_onoff_value(pg2, _("Pixmap Cache (bits)"),
 		   &conf.pixmapcache, &conf.pixmapcachesize,
-		   "5120 Kbits", 5120 * 1024,
-		   "10 Mbits", 10240 * 1024,
-		   "20 Mbits", 20 * 1024 * 1024,
-		   "40 Mbits", 40 * 1024 * 1024,
-		   "60 Mbits", 60 * 1024 * 1024,
-		   "100 Mbits", 100 * 1024 * 1024);
-   add_onoff_visual(pg3, "Force Visual ID");
+		   _("5120 Kbits"), 5120 * 1024,
+		   _("10 Mbits"), 10240 * 1024,
+		   _("20 Mbits"), 20 * 1024 * 1024,
+		   _("40 Mbits"), 40 * 1024 * 1024,
+		   _("60 Mbits"), 60 * 1024 * 1024,
+		   _("100 Mbits"), 100 * 1024 * 1024);
+   add_onoff_visual(pg3, _("Force Visual ID"));
    add_sliders(pg4);
    add_loadsave();
    ready();
